import { Keypair, Transaction, } from '@solana/web3.js';
import { toChainId, toNative, } from '@wormhole-foundation/connect-sdk';
import { SolanaAddress, SolanaPlatform, SolanaUnsignedTransaction, } from '@wormhole-foundation/connect-sdk-solana';
import { createPostVaaInstruction, createReadOnlyWormholeProgramInterface, createVerifySignaturesInstructions, } from './utils';
export class SolanaWormholeCore {
    network;
    chain;
    connection;
    contracts;
    chainId;
    coreBridge;
    address;
    constructor(network, chain, connection, contracts) {
        this.network = network;
        this.chain = chain;
        this.connection = connection;
        this.contracts = contracts;
        this.chainId = toChainId(chain);
        const coreBridgeAddress = contracts.coreBridge;
        if (!coreBridgeAddress)
            throw new Error(`CoreBridge contract Address for chain ${chain} not found`);
        this.address = coreBridgeAddress;
        this.coreBridge = createReadOnlyWormholeProgramInterface(coreBridgeAddress, connection);
    }
    static async fromRpc(connection, config) {
        const [network, chain] = await SolanaPlatform.chainFromRpc(connection);
        const conf = config[chain];
        if (conf.network !== network)
            throw new Error(`Network mismatch for chain ${chain}: ${conf.network} != ${network}`);
        return new SolanaWormholeCore(network, chain, connection, config[chain].contracts);
    }
    publishMessage(sender, message) {
        throw new Error('Method not implemented.');
    }
    async *postVaa(sender, vaa, blockhash) {
        const senderAddr = new SolanaAddress(sender).unwrap();
        const signatureSet = Keypair.generate();
        const verifySignaturesInstructions = await createVerifySignaturesInstructions(this.connection, this.coreBridge.programId, senderAddr, vaa, signatureSet.publicKey);
        // Create a new transaction for every 2 signatures we have to Verify
        for (let i = 0; i < verifySignaturesInstructions.length; i += 2) {
            const verifySigTx = new Transaction().add(...verifySignaturesInstructions.slice(i, i + 2));
            verifySigTx.recentBlockhash = blockhash;
            verifySigTx.feePayer = senderAddr;
            verifySigTx.partialSign(signatureSet);
            yield this.createUnsignedTx(verifySigTx, 'Redeem.VerifySignature', true);
        }
        // Finally create the VAA posting transaction
        const postVaaTx = new Transaction().add(createPostVaaInstruction(this.connection, this.coreBridge.programId, senderAddr, vaa, signatureSet.publicKey));
        postVaaTx.recentBlockhash = blockhash;
        postVaaTx.feePayer = senderAddr;
        yield this.createUnsignedTx(postVaaTx, 'Redeem.PostVAA');
    }
    async parseTransaction(txid) {
        const config = {
            maxSupportedTransactionVersion: 0,
        };
        const response = await this.connection.getTransaction(txid, config);
        if (!response || !response.meta?.innerInstructions[0]?.instructions)
            throw new Error('transaction not found');
        const instructions = response.meta?.innerInstructions[0].instructions;
        const accounts = response.transaction.message.getAccountKeys();
        // find the instruction where the programId equals the Wormhole ProgramId
        const bridgeInstructions = instructions.filter((i) => {
            const programId = accounts.get(i.programIdIndex).toString();
            const wormholeCore = this.coreBridge.programId.toString();
            return programId === wormholeCore;
        });
        if (bridgeInstructions.length === 0)
            throw new Error('no bridge messages found');
        const messagePromises = bridgeInstructions.map(async (bi) => {
            const messageAcct = accounts.get(bi.accounts[1]);
            const emitterAcct = accounts.get(bi.accounts[2]);
            const emitter = toNative(this.chain, emitterAcct.toString());
            const acctInfo = await this.connection.getAccountInfo(messageAcct);
            const sequence = acctInfo.data.readBigUInt64LE(49);
            return {
                chain: this.chain,
                emitter: emitter.toUniversalAddress(),
                sequence: BigInt(sequence),
            };
        });
        return await Promise.all(messagePromises);
    }
    createUnsignedTx(txReq, description, parallelizable = false) {
        return new SolanaUnsignedTransaction(txReq, this.network, this.chain, description, parallelizable);
    }
}
//# sourceMappingURL=core.js.map